import sys
import argparse
import pluggy
import signal
import logging

from .. import __data__
from . import hookspec
from . import default_plugins
from . import app
from . lib.logger import logger
from . lib.safe_print import safe_print


def run_awsume(argument_list):
    awsume = app.Awsume()
    awsume.run(argument_list)


def main():
    try:
        if '--debug' in sys.argv:
            logger.setLevel(logging.DEBUG)
            logger.debug('Debug logs are visible')
        elif '--info' in sys.argv:
            logger.setLevel(logging.INFO)
            logger.info('Info logs are visible')
        logger.debug('Executing awsume')
        run_awsume(sys.argv[1:])
    except KeyboardInterrupt:
        pass



import pluggy
import argparse

hookspec = pluggy.HookspecMarker('awsume')


@hookspec
def pre_add_arguments(config: dict):
    """pre add_arguments hook"""

@hookspec
def add_arguments(config: dict, parser: argparse.ArgumentParser):
    """add argparse arguments to the parser, should try/except for conflicting arguments"""

@hookspec
def post_add_arguments(config: dict, arguments: argparse.Namespace, parser: argparse.ArgumentParser):
    """post add_arguments hook"""



@hookspec
def pre_collect_aws_profiles(config: dict, arguments: argparse.Namespace, credentials_file: str, config_file: str):
    """"""

@hookspec
def collect_aws_profiles(config: dict, arguments: argparse.Namespace, credentials_file: str, config_file: str):
    """"""

@hookspec
def post_collect_aws_profiles(config: dict, arguments: argparse.Namespace, profiles: dict):
    """"""



@hookspec
def pre_get_credentials(config: dict, arguments: argparse.Namespace, profiles: dict):
    """"""

@hookspec
def get_credentials(config: dict, arguments: argparse.Namespace, profiles: dict):
    """"""

@hookspec
def get_credentials_with_saml(config: dict, arguments: argparse.Namespace):
    """"""

@hookspec
def get_credentials_with_web_identity(config: dict, arguments: argparse.Namespace):
    """"""

@hookspec
def post_get_credentials(config: dict, arguments: argparse.Namespace, profiles: dict, credentials: dict):
    """"""



@hookspec
def catch_profile_not_found_exception(config: dict, arguments: argparse.Namespace, profiles: dict, error: Exception):
    """"""

@hookspec
def catch_invalid_profile_exception(config: dict, arguments: argparse.Namespace, profiles: dict, error: Exception):
    """"""

@hookspec
def catch_user_authentication_error(config: dict, arguments: argparse.Namespace, profiles: dict, error: Exception):
    """"""

@hookspec
def catch_role_authentication_error(config: dict, arguments: argparse.Namespace, profiles: dict, error: Exception):
    """"""



@hookspec
def get_profile_names(config: dict, arguments: argparse.Namespace):
    """"""



import os
import sys

import colorama
import yaml
from colorama import init

from . constants import AWSUME_CONFIG


def safe_print(message: str, color: str = '', end: str = None):
    """Safely print so no data is interfering with the shell wrapper"""
    config = yaml.safe_load(open(str(AWSUME_CONFIG), 'r')) or {}
    if not config:
        config = {'colors': True}
    if os.name == 'nt' or config.get('colors') != True:
        color = ''
    print(str(color) + str(message) + colorama.Style.RESET_ALL, end=end, file=sys.stderr)



from pathlib import Path

AWSUME_DIR = Path('~/.awsume').expanduser()
AWSUME_CONFIG = Path('~/.awsume' + '/config.yaml').expanduser()

DEFAULT_CREDENTIALS_FILE = Path('~/.aws/credentials').expanduser()
DEFAULT_CONFIG_FILE = Path('~/.aws/config').expanduser()

AWSUME_CACHE_DIR = Path('~/.awsume/cache').expanduser()



import os, subprocess, sys
from pathlib import Path
from setuptools.command.install import install
from distutils.spawn import find_executable

from .main import run

BASH_LOGIN_FILES = ['~/.bash_profile', '~/.bash_login', '~/.profile', '~/.bashrc']


class CustomInstall(install):
    def get_bash_file(self) -> str:
        paths = [str(Path(_).expanduser()) for _ in BASH_LOGIN_FILES]
        result = [_ for _ in paths if os.path.exists(_) and os.path.isfile(_) and os.access(_, os.R_OK)]
        if not result:
            default = paths[0]
            open(default, 'w').close()
            result = [default]
        return result[0]

    def get_zsh_file(self) -> str:
        z_dot_dir = os.environ.get('ZDOTDIR', '~')
        zsh_file = str(Path(z_dot_dir + '/.zshenv').expanduser())
        if not os.path.exists(zsh_file) or not os.path.isfile(zsh_file):
            open(zsh_file, 'w').close()
        return zsh_file

    def run(self):
        install.run(self)
        if os.environ.get('AWSUME_SKIP_ALIAS_SETUP'):
            print('===== Skipping Alias Setup =====')
            return
        if find_executable('bash'):
            print('===== Setting up bash =====')
            bash_file = self.get_bash_file()
            run('bash', bash_file, bash_file)
        if find_executable('zsh'):
            print('===== Setting up zsh =====')
            zsh_file = self.get_zsh_file()
            run('zsh', zsh_file, zsh_file)
        if find_executable('powershell'):
            print('===== Setting up powershell =====')
            (powershell_file, _) = subprocess.Popen(['powershell', 'Write-Host $profile'], stdout=subprocess.PIPE, shell=True).communicate()
            if powershell_file:
                powershell_file = str(powershell_file.decode('ascii')).replace('\r\n', '').replace('\n', '')
                run('powershell', None, powershell_file)
            else:
                print('===== Could not locate powershell file =====')
        print('===== Finished setting up =====', file=sys.stderr)
